/** Example 013 Render To Texture

This tutorial shows how to render to a texture using Irrlicht. Render to
texture is a feature with which it is possible to create nice special effects.
In addition, this tutorial shows how to enable specular highlights.

In the beginning, everything as usual. Include the needed headers, ask the user
for the rendering driver, create the Irrlicht Device:
*/

#include <irrlicht.h>
#include <iostream>
#include "driverChoice.h"
#include "postprocessing/CScreenQuad.hpp"
#include "postprocessing/CScreenQuadSceneNode.hpp"
#include "postprocessing/CBlurShader.hpp"

using namespace irr;
using namespace video;

#ifdef _MSC_VER
#pragma comment(lib, "Irrlicht.lib")
#endif


const u32 width = 640;
const u32 height = 480;

int main()
{

    CScreenQuad quad;
    CScreenQuadSceneNode* node = 0;
    CBlurShader* HBlur = 0;
    CBlurShader* VBlur = 0;

	// ask user for driver
	video::E_DRIVER_TYPE driverType = video::EDT_OPENGL;

//	driverType= driverChoiceConsole();

	if (driverType==video::EDT_COUNT)
		return 1;

	// create device and exit if creation failed

	IrrlichtDevice *device =
		createDevice(driverType, core::dimension2d<u32>( width, height),
		16, false, false);

	if (device == 0)
		return 1; // could not create selected driver.

	video::IVideoDriver* driver = device->getVideoDriver();
	scene::ISceneManager* smgr = device->getSceneManager();
	gui::IGUIEnvironment* env = device->getGUIEnvironment();
    ILogger* logger = device->getLogger();

    logger->setLogLevel(ELL_INFORMATION );
	/*
	Now, we load an animated mesh to be displayed. As in most examples,
	we'll take the fairy md2 model. The difference here: We set the
	shininess of the model to a value other than 0 which is the default
	value. This enables specular highlights on the model if dynamic
	lighting is on. The value influences the size of the highlights.
	*/



		const E_VERTEX_SHADER_TYPE vertexProfile =
			driver->queryFeature(video::EVDF_VERTEX_SHADER_3_0) ? EVST_VS_3_0 : EVST_VS_2_0;

		const E_PIXEL_SHADER_TYPE pixelProfile =
			driver->queryFeature(video::EVDF_PIXEL_SHADER_3_0) ? EPST_PS_3_0 : EPST_PS_2_0;


	// load and display animated fairy mesh

    node = new CScreenQuadSceneNode(0,smgr, -1);

	scene::IAnimatedMeshSceneNode* fairy = smgr->addAnimatedMeshSceneNode(
		smgr->getMesh("../../media/faerie.md2"));

	if (fairy)
	{
		fairy->setMaterialTexture(0,
				driver->getTexture("../../media/faerie2.bmp")); // set diffuse texture
		fairy->setMaterialFlag(video::EMF_LIGHTING, true); // enable dynamic lighting
		fairy->getMaterial(0).Shininess = 20.0f; // set size of specular highlights
		fairy->setPosition(core::vector3df(-10,0,-100));
		fairy->setMD2Animation ( scene::EMAT_STAND );
	}

	/*
	To make specular highlights appear on the model, we need a dynamic
	light in the scene. We add one directly in vicinity of the model. In
	addition, to make the model not that dark, we set the ambient light to
	gray.
	*/

	// add white light
	smgr->addLightSceneNode(0, core::vector3df(-15,5,-105),
			video::SColorf(1.0f, 1.0f, 1.0f));

	// set ambient light
	smgr->setAmbientLight(video::SColor(0,60,60,60));

	/*
	The next is just some standard stuff: Add a test cube and let it rotate
	to make the scene more interesting. The user defined camera and cursor
	setup is made later on, right before the render loop.
	*/

	// create test cube
//	scene::ISceneNode* test = smgr->addCubeSceneNode(60);
//
//	// let the cube rotate and set some light settings
//	scene::ISceneNodeAnimator* anim = smgr->createRotationAnimator(
//		core::vector3df(0.3f, 0.3f,0));
//
//	test->setPosition(core::vector3df(-100,0,-100));
//	test->setMaterialFlag(video::EMF_LIGHTING, false); // disable dynamic lighting
//	test->addAnimator(anim);
//	test->setVisible(false);
//
//	anim->drop();

	// set window caption
	device->setWindowCaption(L"Irrlicht Engine - Render to Texture and Specular Highlights example");

	/*
	To test out the render to texture feature, we need a render target
	texture. These are not like standard textures, but need to be created
	first. To create one, we call IVideoDriver::addRenderTargetTexture()
	and specify the size of the texture. Please don't use sizes bigger than
	the frame buffer for this, because the render target shares the zbuffer
	with the frame buffer.
	Because we want to render the scene not from the user camera into the
	texture, we add another fixed camera to the scene. But before we do all
	this, we check if the current running driver is able to render to
	textures. If it is not, we simply display a warning text.
	*/

	// create render target
	video::ITexture* rt = 0;
	video::ITexture* rtAfterHBlur = 0;
	video::ITexture* rtAfterVBlur = 0;
	scene::ICameraSceneNode* fixedCam = 0;

    if (!driver->queryFeature(video::EVDF_PIXEL_SHADER_1_1) &&
            !driver->queryFeature(video::EVDF_ARB_FRAGMENT_PROGRAM_1))
    {
            logger->log("WARNING: Pixel shaders disabled "\
                    "because of missing driver/hardware support.");

    }


	if (driver->queryFeature(video::EVDF_RENDER_TO_TARGET))
	{
	    logger->log("REnder to target supported");
		rt = driver->addRenderTargetTexture(core::dimension2d<u32>(width,height), "RTT1");
//		test->setMaterialTexture(0, rt); // set material of cube to render target

        rtAfterHBlur = driver->addRenderTargetTexture(core::dimension2d<u32>(width,height), "HBlur");
        rtAfterVBlur = driver->addRenderTargetTexture(core::dimension2d<u32>(width,height), "VBlur");

        if(rtAfterHBlur == 0)
        {

            logger->log("Could not create render texture");
        }
		// add fixed camera
		fixedCam = smgr->addCameraSceneNode(0, core::vector3df(10,10,-80),
			core::vector3df(-10,10,-100));
	}
	else
	{
		// create problem text
		gui::IGUISkin* skin = env->getSkin();
		gui::IGUIFont* font = env->getFont("../../media/fonthaettenschweiler.bmp");
		if (font)
			skin->setFont(font);

		gui::IGUIStaticText* text = env->addStaticText(
			L"Your hardware or this renderer is not able to use the "\
			L"render to texture feature. RTT Disabled.",
			core::rect<s32>(150,20,470,60));

		text->setOverrideColor(video::SColor(100,255,255,255));
	}

	// add fps camera
	scene::ICameraSceneNode* fpsCamera = smgr->addCameraSceneNodeFPS();
	fpsCamera->setPosition(core::vector3df(-50,50,-150));

	// disable mouse cursor
	//device->getCursorControl()->setVisible(true);

	/*
	Nearly finished. Now we need to draw everything. Every frame, we draw
	the scene twice. Once from the fixed camera into the render target
	texture and once as usual. When rendering into the render target, we
	need to disable the visibilty of the test cube, because it has the
	render target texture applied to it. That's it, wasn't too complicated
	I hope. :)
	*/

	int lastFPS = -1;


    video::IGPUProgrammingServices* gpu = driver->getGPUProgrammingServices();


    // Creates callbacks
    HBlur = new CBlurShader("hello world");


    // File / Entry point / vsCOmpileTarget
    s32 BlurHP = 0;
    s32 BlurVP = 0;

    BlurHP  = gpu->addHighLevelShaderMaterialFromFiles(
                    "shaders/default.vert.glsl", "main",vertexProfile,
//                    "shaders/default.frag.glsl", "main", pixelProfile
                    "shaders/BlurHP.perso.glsl", "main", pixelProfile
                    , HBlur
                    , video::EMT_SOLID
                    );

    BlurVP  = gpu->addHighLevelShaderMaterialFromFiles(
                    "shaders/default.vert.glsl", "main",vertexProfile,
//                    "shaders/default.frag.glsl", "main", pixelProfile
                    "shaders/BlurVP.perso.glsl", "main", pixelProfile
                    , VBlur
                    , video::EMT_SOLID
                    );



	while(device->run())
	if (device->isWindowActive())
	{
		driver->beginScene(true, true, 0);

		if (rt)
		{

			// set render target texture
			driver->setRenderTarget(rt, true, true, video::SColor(0,255,255,255));

			// make cube invisible and set fixed camera as active camera

//			smgr->setActiveCamera( fpsCamera ); //fixedCam
			smgr->setActiveCamera( fixedCam );

			// draw whole scene into render buffer
			smgr->drawAll();


			driver->setRenderTarget(rtAfterHBlur, true, true,  video::SColor(0,0,0,255));

//            node->getMaterial(0).MaterialType = EMT_SOLID;
            node->getMaterial(0).MaterialType = (E_MATERIAL_TYPE)BlurHP;
            node->getMaterial(0).Lighting = false;
            node->getMaterial(0).setTexture(0,rt);
//            node->setMaterialTexture(0,rt);
            node->render();



//            driver->setRenderTarget(0, true, true,  video::SColor(0,255,255,255));
            driver->setRenderTarget(0);

            node->getMaterial(0).MaterialType = EMT_SOLID;
//            node->getMaterial(0).setTexture(0,rtAfterHBlur);
            node->getMaterial(0).setTexture(0,rtAfterHBlur);
            node->render();

		}


        driver->enableMaterial2D();
        driver->draw2DImage( rt, core::rect<s32>( core::vector2di(0,0),core::dimension2di(100,100) ),
                core::rect<s32>(0,0,width,height));

        driver->draw2DImage( rtAfterHBlur, core::rect<s32>( core::vector2di(100,0),core::dimension2di(100,100) ),
                core::rect<s32>(0,0,width,height));

        driver->enableMaterial2D(false);

		env->drawAll();

		driver->endScene();

		// display frames per second in window title
		int fps = driver->getFPS();
		if (lastFPS != fps)
		{
			core::stringw str = L"Irrlicht Engine - Render to Texture and Specular Highlights example";
			str += " FPS:";
			str += fps;

			device->setWindowCaption(str.c_str());
			lastFPS = fps;
		}
	}

	device->drop(); // drop device
	return 0;
}

/*
**/
